<?php
/**
 *
 * phpBB WordPress integration. An extension for the phpBB Forum Software package.
 *
 * @copyright (c) 2022, axew3, http://axew3.com
 * @license GNU General Public License, version 2 (GPL-2.0)
 *
 */

namespace w3all\phpbbwordpress\controller;

/**
 * Display Only First Post ACP controller.
 */
class acp_controller
{

  protected $config;
  protected $config_text;
  protected $table_prefix;
  protected $db;
  protected $language;
  protected $log;
  protected $request;
  protected $template;
  protected $user;
  protected $u_action;

  /**
   * Constructor.
   */
  public function __construct(\phpbb\config\config $config, \phpbb\config\db_text $config_text, $table_prefix, \phpbb\db\driver\factory $db, \phpbb\language\language $language, \phpbb\log\log $log, \phpbb\request\request $request, \phpbb\template\template $template, \phpbb\user $user)
  {
    $this->config = $config;
    $this->config_text = $config_text;
    $this->table_prefix = $table_prefix;
    $this->db = $db;
    $this->language = $language;
    $this->log = $log;
    $this->request = $request;
    $this->template = $template;
    $this->user = $user;
  }

  /**
   * Display the options a user can configure for this extension.
   *
   * @return void
   */
  public function display_options()
  {
    // Add our common language file
    $this->language->add_lang('common', 'w3all/phpbbwordpress');

    // Create a form key for preventing CSRF attacks
    add_form_key('w3all_phpbbwordpress');

    // Create an array to collect errors that will be output to the user
    $errors = [];

    // Is the form being submitted to us?
    if ($this->request->is_set_post('submit'))
    {
      // Test if the submitted form is valid
      if (!check_form_key('w3all_phpbbwordpress'))
      {
        $errors[] = $this->language->lang('FORM_INVALID');
      }

      // If no errors, process the form data
      if (empty($errors))
      {

        $POST_data = $this->request->get_super_global(\phpbb\request\request_interface::POST);

        if(!empty($POST_data['w3all_phpbbwordpress']))
        {
          // test db connection data, store values if no error
          $db = new \phpbb\db\driver\mysqli();
          if(! $db->sql_connect($POST_data['w3all_phpbbwordpress']['wpdbhost'], $POST_data['w3all_phpbbwordpress']['wpdbuser'], $POST_data['w3all_phpbbwordpress']['wpdbpass'], $POST_data['w3all_phpbbwordpress']['wpdbname'], $POST_data['w3all_phpbbwordpress']['wpdbport'], false, false))
         { // if no connection, phpBB MySQL lib will fall into error here, so no wrong db data will be stored: anyway, if any future phpBB changes, this will ever work..
         	 die("<h1>Wrong Connection values. Return back.</h1>");
          }

         $w3all_ary = array('w3all_phpbbwordpress' => $POST_data['w3all_phpbbwordpress'], 'w3all_phpbbwordpress_opt' => $POST_data['w3all_phpbbwordpress_opt']);
         $this->config_text->set('w3all_phpbbwordpress', serialize($w3all_ary));
        }

        // Add option settings change action to the admin log
        $this->log->add('admin', $this->user->data['user_id'], $this->user->ip, 'LOG_ACP_DISPLAYONLYFIRSTPOST_SETTINGS');

        // Option settings have been updated and logged
        // Confirm and provide link back to previous page
        trigger_error($this->language->lang('ACP_PHPBBWORDPRESS_SETTING_SAVED') . adm_back_link($this->u_action));
      }
    }

    $s_errors = !empty($errors);
    $data = $this->config_text->get('w3all_phpbbwordpress');

    if(empty($data)){
     $dbcdata = array();
     $dbcdata['wpdbhost']=$dbcdata['wpdbname']=$dbcdata['wpdbuser']=$dbcdata['wpdbpass']=$dbcdata['wpdbtableprefix']=$dbcdata['wpdbport']=$dbcdata['wpurl']=$optdata['u_add_inwp']=$optdata['ck_u']=$optdata['ck_u_bruteremove']=$optdata['u_redir_onautolog']=$optdata['u_redir_afterlog']='';
    } else {
      $data = unserialize($data);
      $dbcdata = $data['w3all_phpbbwordpress'];
      $optdata = $data['w3all_phpbbwordpress_opt'];
    }

  // Set output variables for display in the template
    $this->template->assign_vars([
      'S_ERROR' => $s_errors,
      'ERROR_MSG' => $s_errors ? implode('<br />', $errors) : '',
      'U_ACTION' => $this->u_action,
      'W3ALL_PHPBBWORDPRESS_WPDBHOST' => $dbcdata['wpdbhost'],
      'W3ALL_PHPBBWORDPRESS_WPDBNAME' => $dbcdata['wpdbname'],
      'W3ALL_PHPBBWORDPRESS_WPDBUSER' => $dbcdata['wpdbuser'],
      'W3ALL_PHPBBWORDPRESS_WPDBPASS' => $dbcdata['wpdbpass'],
      'W3ALL_PHPBBWORDPRESS_WPDBTABLEPREFIX' => $dbcdata['wpdbtableprefix'],
      'W3ALL_PHPBBWORDPRESS_WPDBPORT' => $dbcdata['wpdbport'],
      'W3ALL_PHPBBWORDPRESS_WPURL' => $dbcdata['wpurl'],

      'W3ALL_PHPBBWORDPRESS_UADD_INWP' => $optdata['u_add_inwp'],
      'W3ALL_PHPBBWORDPRESS_CK_U' => $optdata['ck_u'],
      'W3ALL_PHPBBWORDPRESS_CK_UBRUTEREMOVE' => $optdata['ck_u_bruteremove'],
      'W3ALL_PHPBBWORDPRESS_UREDIR_ONAUTOLOG' => $optdata['u_redir_onautolog'],
      'W3ALL_PHPBBWORDPRESS_UREDIR_AFTERLOG' => $optdata['u_redir_afterlog'],
    ]);
  }

  /**
   * Set custom form action.
   *
   * @param string  $u_action Custom form action
   * @return void
   */
  public function set_page_url($u_action)
  {
    $this->u_action = $u_action;
  }
}
