<?php
/**
 *
 * phpBB WordPress Integration Common Tasks. An extension for the phpBB Forum Software package.
 *
 * @copyright (c) 2023 axew3.com
 *
 */

namespace w3all\phpbbwordpress\event;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * ACP Event listener
 */
class acp_listener implements EventSubscriberInterface
{

  /** @var string $phpbb_root_path phpBB root path */

  protected $config_text;
  protected $db;
  protected $phpbb_root_path;
  protected $request;
  protected $table_prefix;
  protected $user;

    public function __construct(\phpbb\config\db_text $config_text, \phpbb\db\driver\factory $db, $phpbb_root_path, \phpbb\request\request_interface $request, $table_prefix, \phpbb\user $user)
  {
    $this->config_text = $config_text;
    $this->db = $db;
    $this->phpbb_root_path = $phpbb_root_path;
    $this->request = $request;
    $this->table_prefix = $table_prefix;
    $this->user = $user;
    $this->wpd = $this->wpdopt = '';

    $wphpbb = unserialize($this->config_text->get('w3all_phpbbwordpress'));

    if(!empty($wphpbb))
    {
      $this->wpd = $wphpbb['w3all_phpbbwordpress'];
      $this->wpdopt = $wphpbb['w3all_phpbbwordpress_opt'];
      include_once($this->phpbb_root_path.'ext/w3all/phpbbwordpress/common/functions_common.php');
      $this->wpdb = wpquery($sql='', $mode='', $this->wpd);
    } else { return; }
  }

  /**
   * {@inheritDoc}
   */
  public static function getSubscribedEvents()
  {
    return array(
      'core.acp_users_overview_modify_data' => 'acp_users_overview_modify_data',
      'core.acp_users_profile_modify_sql_ary' => 'acp_users_profile_modify_sql_ary',
    );
  }

  public function acp_users_overview_modify_data($event)
  {

   // $event['user_row']['user_email'] = old or actual email
   // $event['data']['email'] = new email or actual email

   // if there is some form error, like pass mismatch
   // phpBB return error before this event, so the follow will never fire

   if($event['data']['email'] != $event['user_row']['user_email'] OR !empty($event['data']['new_password']) && !empty($event['data']['password_confirm']))
   {

    if(!empty($event['data']['new_password']))
    {
      $new_password = trim($event['data']['new_password']);
      $password = stripslashes(htmlspecialchars($new_password, ENT_COMPAT));
      $new_password = password_hash($password, PASSWORD_BCRYPT,['cost' => 12]); // phpBB min cost 12
      $newpQ = "user_pass = '". $new_password ."',";
    } else {
      $newpQ = '';
    }

    $sql = "UPDATE ".$this->wpd['wpdbtableprefix']."users SET ".$newpQ." user_email = '". $this->db->sql_escape(strtolower($event['data']['email'])) ."' WHERE LOWER(user_email) = '". strtolower($event['user_row']['user_email']) ."'";
    $this->wpdb = wpquery($sql,'update',$this->wpd);

   }

  }


    public function acp_users_profile_modify_sql_ary($e)
  {
    $sql = "UPDATE ".$this->wpd['wpdbtableprefix']."users SET user_url = '". $this->db->sql_escape($e['cp_data']['pf_phpbb_website']) ."' WHERE LOWER(user_email) = '". strtolower($e['user_row']['user_email']) ."'";
    $res = $this->wpdb = wpquery($sql,'update',$this->wpd);
  }

}
