<?php
/**
 *
 * phpBB WordPress Integration Common Tasks. An extension for the phpBB Forum Software package.
 *
 * @copyright (c) 2022, axew3 - axew3.com
 * @license GNU General Public License, version 2 (GPL-2.0)
 *
 */

namespace w3all\phpbbwordpress\event;

/**
 * @ignore
 */
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * phpBB WordPress Integration Common Tasks Event listener.
 */
class main_listener implements EventSubscriberInterface
{
  protected $config;
  protected $config_text;
  protected $phpbb_root_path;
  protected $table_prefix;
  protected $db;
  protected $request;
  protected $template;
  protected $user;

  public function __construct(\phpbb\config\config $config, \phpbb\config\db_text $config_text, $phpbb_root_path, $table_prefix, \phpbb\db\driver\factory $db, \phpbb\request\request_interface $request, \phpbb\template\template $template, \phpbb\user $user)
  {

    $this->config = $config;
    $this->config_text = $config_text;
    $this->db = $db;
    $this->phpbb_root_path = $phpbb_root_path;
    $this->request = $request;
    $this->table_prefix = $table_prefix;
    $this->template = $template;
    $this->user = $user;
    $this->wpd = $this->wpdopt = $this->redir_wp = false;

    $wphpbb = unserialize($this->config_text->get('w3all_phpbbwordpress'));

    if(!empty($wphpbb))
    {
      $this->wpd = $wdp = $wphpbb['w3all_phpbbwordpress'];
      $this->wpdopt = $wpdopt = $wphpbb['w3all_phpbbwordpress_opt'];
      include_once($this->phpbb_root_path.'ext/w3all/phpbbwordpress/common/functions_common.php');
  	  $this->wpdb = wpquery($sql='', $mode='', $this->wpd);
    } else { return; }
   }


  public static function getSubscribedEvents()
  {

    return array(
      'core.common' => 'common_mimic_endpoints',
      //'core.delete_user_after' => 'delete_user_after',
      //'core.delete_user_before' => 'delete_user_before',
      //'core.login_box_redirect' => 'login_box_redirect',
      'core.session_create_after' => 'session_create_after',
      'core.ucp_activate_after' => 'ucp_activate_after', // Note: email update/change in WordPress, should instead happen when email successfully confirmed (like WP do)?
      'core.ucp_profile_info_modify_sql_ary' => 'ucp_profile_info_modify_sql_ary',
      'core.ucp_profile_reg_details_validate' => 'ucp_profile_reg_details_validate',
      'core.ucp_register_data_after' => 'ucp_register_data_after',
      'core.user_add_modify_data' => 'user_add_modify_data',
      'core.user_add_after' => 'user_add_after',
    );
  }


// check that the username or email do not exist into the linked WordPress, before the user addition into phpBB
     public function ucp_register_data_after($e)
  {

   if( empty($e['error']) && !empty($this->wpdopt['ck_u']) )
   {

      // phpBB configuration should be set to allow only spaces, a-zA-z0-9 range and _.@ chars, and usernames lenght limit to 50 chars
      // check if it is not
      // max username lenght in wp: 50 , range of allowed usernames chars in wp: '-0-9A-Za-z _.@'
      // NOTE that WP Multisite only allow '0-9A-Za-z' range for usernames (no spaces nor -_.@ chars)

      $contains_cyrillic = (bool) preg_match('/[\p{Cyrillic}]/u', $e['data']['username']);
      if ( $contains_cyrillic && preg_match('/[^-0-9A-Za-z _.@\p{Cyrillic}]/u',$e['data']['username']) OR preg_match('/[^-0-9A-Za-z _.@]/',$e['data']['username']) OR strlen($e['data']['username']) > 50 )
      {
       $e['error'] = [ 0 => 'Username containing illegal characters for the linked CMS or there are too many characters. Change the username.' ];
        return;
      } elseif ( !filter_var($e['data']['email'], FILTER_VALIDATE_EMAIL) ) {
       $e['error'] = [ 0 => 'Wrong email format.' ];
        return;
      }

      $sql = "SELECT * FROM ".$this->wpd['wpdbtableprefix']."users WHERE user_login = '". $this->db->sql_escape($e['data']['username']) ."' OR LOWER(user_email) = '". $this->db->sql_escape(strtolower($e['data']['email'])) ."'";
      $wpu = $this->wpdb = wpquery($sql,'select',$this->wpd);

       if(!empty($wpu))
       {
         $e['error'] = [ 0 => 'The username or email exists into the linked CMS.' ];
       }
     }

   }


// redirect after a phpBB login
/*     public function login_box_redirect($event)
  {

    if( $this->wpdopt['u_redir_afterlog'] > 0 ){
      global $phpbb_adm_relative_path; // detect (and exclude) redirect if accessing phpBB admin
      if( false === strpos( $this->request->server('REQUEST_URI'), $phpbb_adm_relative_path ) && $this->request->variable('mode', '') != 'logout' )
      {
       header("Location:".$this->wpd['wpurl']); exit;
      }
     }
   }*/

// Clean up the wp bruteforce record for this uid
// Redirect after phpBB session setup, if it is a login or $this->redir_wp case
     public function session_create_after($e)
  {
  	// wp bruteforce cleanup
  	if($this->request->variable('mode','') == 'login'){
  	 $sql = "SELECT option_value FROM ".$this->wpd['wpdbtableprefix']."options WHERE option_name = 'w3all_bruteblock_phpbbulist'";
     $wpBF_ary = $this->wpdb = wpquery($sql,'select',$this->wpd);

     if(!empty($wpBF_ary)){
      $wpBF_ary = unserialize($wpBF_ary['option_value']);

       if(isset($this->user->data['user_id']) && isset($wpBF_ary[$this->user->data['user_id']])){

      	 unset($wpBF_ary[$this->user->data['user_id']]);
      	 $wpbf = !empty($wpBF_ary) ? serialize($wpBF_ary) : '';
         $sql = "UPDATE ".$this->wpd['wpdbtableprefix']."options SET option_value = '". $wpbf ."' WHERE option_name = 'w3all_bruteblock_phpbbulist'";
         $this->wpdb = wpquery($sql,'update',$this->wpd);
       }
     }
    } // END wp bruteforce cleanup

    if(!empty($this->wpd['wpurl']))
    {
     if( $this->redir_wp == true OR $this->wpdopt['u_redir_afterlog'] > 0 && $this->request->variable('mode','') == 'login' OR $this->wpdopt['u_redir_onautolog'] > 0 && $this->request->variable('mode', '') == 'login' )
     {
      global $phpbb_adm_relative_path; // detect (and exclude) redirect if accessing phpBB admin
      if( false === strpos( $this->request->server('REQUEST_URI'), $phpbb_adm_relative_path ) )
      {
       header("Location:".$this->wpd['wpurl']); exit;
      }
     }
    }
   }

// if updating when on ucp 'Edit profile'
   public function ucp_profile_info_modify_sql_ary($event)
  {

     if( empty($event['cp_data']['pf_phpbb_website']) OR $this->user->data['user_id'] < 3 )
     {
      return;
     }

    if( ! empty($event['cp_data']['pf_phpbb_website']) )
    {
     $sql = "UPDATE ".$this->wpd['wpdbtableprefix']."users SET user_url = '". $this->db->sql_escape($event['cp_data']['pf_phpbb_website']) ."' WHERE LOWER(user_email) = '". strtolower($this->user->data['user_email']) ."'";
     $this->wpdb = wpquery($sql,'update',$this->wpd);
    }
  }

// if updating when on ucp 'Edit account setting'
   public function ucp_profile_reg_details_validate($event)
  {
     if( !empty($event['error']) OR $this->user->data['user_id'] < 3 ){ // exclude phpBB id2
      return;
     }

    // $user->data['user_email'] = old or actual email
    // $event['data']['email'] = new email or actual email

    if($event['data']['email'] != $this->user->data['user_email'] OR !empty($event['data']['new_password']) && !empty($event['data']['password_confirm']) )
    {

     if(!empty($event['data']['new_password']))
     {
       $new_password = trim($event['data']['new_password']);
       $password = stripslashes(htmlspecialchars($new_password, ENT_COMPAT));
       $new_password = password_hash($password, PASSWORD_BCRYPT,['cost' => 12]); // phpBB min cost 12
       $newp = "user_pass = '". $new_password ."',";
      } else { $newp = ''; }
       $sql = "UPDATE ".$this->wpd['wpdbtableprefix']."users SET ".$newp." user_email = '". $this->db->sql_escape(strtolower($event['data']['email'])) ."' WHERE LOWER(user_email) = '". strtolower($this->user->data['user_email']) ."'";
       $this->wpdb = wpquery($sql,'update',$this->wpd);
     }
   }

   public function user_add_modify_data($e)
  {

    // the user's 'reset_token' db field is not set into the array of data for the being created new user:
    // so it is used to store a token that then can be checked for a legit WP user insertion
    // or something else. // db field limit 64 chars

    $token = str_shuffle(  bin2hex(random_bytes((rand(10,30)))) . strtoupper(bin2hex(random_bytes((rand(10,20))))) );

    if( strlen($token) > 64 )
    {
     $token = substr($token, -mt_rand(50,64));
    }

    $e['sql_ary'] += [ "reset_token" => $token ];
    $this->user->data['reset_token'] = $token;
  }

// if account require activation
     public function ucp_activate_after($e)
  {
     // ACCOUNT_ACTIVE deactivated due to registration // email verification confirmed account activated
     // ACCOUNT_ACTIVE_PROFILE deactivated due to email change on profile // email change/verification confirmed

     // if( $event['message'] == 'ACCOUNT_ACTIVE' OR $event['message'] == 'ACCOUNT_ACTIVE_PROFILE' ){

    if( $e['message'] == 'ACCOUNT_ACTIVE' )
    {
     if(self::w3_wp_curl($e['user_row']['user_id'], $e['user_row']['user_email'], $this->wpd['wpurl'], $this->config['avatar_salt'], $this->user->data['reset_token'] ) === true)
     {
       if( $this->wpdopt['u_redir_onautolog'] > 0 )
       {
        if(!empty($this->wpd['wpurl']))
        {
         global $phpbb_adm_relative_path;
         if( false === strpos( $this->request->server('REQUEST_URI'), $phpbb_adm_relative_path ) )
         { header("Location:".$this->wpd['wpurl']); exit; }
        }

       }
     }

   }
  }

// no activation required, immediate access
     public function user_add_after($e)
  {
    // $e['user_row']['user_inactive_reason'] != 0 // the new account require email verification
    // $e['user_id'] // user id

    if( $e['user_row']['user_new'] != 1 OR $e['user_row']['user_inactive_reason'] != 0 )
    {
      return; // user to be added after email confirmation
    }

    if( $e['user_row']['user_inactive_reason'] == 0 && $e['user_row']['user_new'] == 1 && !empty($this->wpd['wpurl']) )
    {

      if(self::w3_wp_curl($e['user_id'], $e['user_row']['user_email'], $this->wpd['wpurl'], $this->config['avatar_salt'], $this->user->data['reset_token']) === true)
      {
        if( $this->wpdopt['u_redir_onautolog'] > 0 )
        {
          $this->redir_wp = true;
        }
      }
    }

  }


     public static function w3_wp_curl($uid, $email, $url = '', $avatar_salt = '', $ureset_token = '')
  {
    if(empty($url)){
      return; // Disable the user addition into wordpress, the extension used/run only for email, pass and url update
    }

   if( !in_array('curl', get_loaded_extensions()) ) {
    return; // cURL not available
   }

    if(!empty($avatar_salt)){
     $tk = stripslashes(htmlspecialchars($avatar_salt, ENT_COMPAT));
     $w3allastoken = password_hash($tk, PASSWORD_BCRYPT,['cost' => 12]);
    } elseif (!empty($ureset_token)){
      $w3allastoken = $ureset_token; // not used because require a query in wp to retrieve user's data for the check: the previous is faster because phpBB config values are already available into WP
     } else { return false; }

      $data = array( 'w3alladdphpbbuid' => $uid, 'w3alladdphpbbuemail' => $email, 'w3allastoken' => $w3allastoken );
      $data = http_build_query($data);
       $ch = curl_init();

      curl_setopt($ch, CURLOPT_URL,$url);
      curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
      curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
      curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: application/x-www-form-urlencoded'));
      curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
      curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);

    if(curl_exec($ch) === false){
      curl_close ($ch);
     return false;
    } else {
       curl_close ($ch);
      return true;
    }
  }

/*   public function delete_user_before( $e )
  {
  } */

 //   public function delete_user_after( $mode, $user_ids, $retain_username=true, $user_rows )
 // {
  /* * @event core.delete_user_after
   * @var string  mode        Mode of posts deletion (retain|remove)
   * @var array user_ids      ID(s) of the deleted user(s)
   * @var bool  retain_username   True if username should be retained, false otherwise
   * @var array user_rows     Array containing data of the deleted user(s)
   * @since 3.1.0-a1
   * @changed 3.2.2-RC1 Added user_rows*/

 // }


  //////////////////////////////////////
  // ENDPOINTS from WordPress START

    public function common_mimic_endpoints()
  {
    // to know from where values are coming, how have been generated, and how the logic work, see wp_w3all.php file
    // wp/wp-content/plugins/wp-w3all-phpbb-integration/wp_w3all.php
    // add_action( 'delete_user', 'w3all_usersdata_predelete_in_phpbb_exec', 10, 3);
    // add_action( 'deleted_user', 'w3all_usersdata_deleted_in_phpbb_exec', 10, 3);

    $phpBB_function_endpoint = $this->request->variable('w3all_phpBB_function_endpoint', '', true);
    if ( empty($phpBB_function_endpoint) ) { return; }

     $tk = stripslashes(htmlspecialchars($this->config['avatar_salt'], ENT_COMPAT));
     $tk = substr(md5($tk), 4, -8);
     $tk0 = substr(strtoupper(md5($tk)), 0, -16);
     $tk .= $tk0;

    $ckv = 'w3all_phpBB_function_endpoint___'.$tk0;
    $w3all_phpBB_function_endpoint_ck_randvar = $this->request->variable($ckv, '', true);
    // first check
    if(empty($w3all_phpBB_function_endpoint_ck_randvar)){ return; }

    // check also that the request come with valid token or avoid to follow
    $w3allastoken = $this->request->variable('w3allastoken', '', true);
    $w3allastoken = str_replace(chr(0), '', $w3allastoken);

      if( !password_verify($tk, $w3allastoken) )
       { // stop any going on here, something goes wrong. An antibruteforce could be easily added for further security.
         // But it is may not necessary, since w3all_usersdata_deleted_in_phpbb_exec() function into wp_w3all.php, empty the value of the relate user's meta db table, just after/when the first user's deletion in WP happen (so just after phpBB received and processed the $_POST here)
         // so just
         return;
        }


   ##### USER DELETION

    if( $phpBB_function_endpoint == 'user_delete' )
    {
      $wpdelete_phpbbulist_delby = $this->request->variable('w3all_wpdelete_phpbbulist_delby', '', true);
      $wpdelete_phpbbulist_delby = intval($wpdelete_phpbbulist_delby); // ID of the WP user deleting in WP
      if( $wpdelete_phpbbulist_delby < 1 ){ return; }
      $delmode = $this->request->variable('w3all_delmode', '', true);
      $delmode = ($delmode == 'retain') ? 'retain' : 'remove';

      $w3all_nonce_reqtime_rand = $this->request->variable('w3all_nonce_reqtime_rand', '', true);

      if( empty($w3all_nonce_reqtime_rand) OR preg_match('/[^_0-9A-Za-z]/',$w3all_nonce_reqtime_rand) )
      { return; }

      $sql = "SELECT * FROM ".$this->wpd['wpdbtableprefix']."usermeta WHERE user_id = '". $wpdelete_phpbbulist_delby ."' AND meta_key = 'w3all_wpdelete_phpbbulist_delby'";
      $delete_ary = $this->wpdb = wpquery($sql,'select',$this->wpd);

    if(!empty($delete_ary['meta_value']))
    {
      $del_ary = unserialize($delete_ary['meta_value']);

      if(!is_array($del_ary) OR empty($del_ary)){ return; }
      // to understand this see
      // function w3all_usersdata_predelete_in_phpbb_exec($id, $reassign, $user)
      // into wp_w3all.php file
      $w3all_nonce_reqtime_rand_meta = array_pop($del_ary); // Get and Remove the $w3all_nonce_reqtime_rand array from array (last key)
      // Now check if the whole passed value match
      if(empty($w3all_nonce_reqtime_rand_meta['w3all_nonce_reqtime_rand']) OR $w3all_nonce_reqtime_rand_meta['w3all_nonce_reqtime_rand'] != $w3all_nonce_reqtime_rand)
      { return; }
      $post_nonce_reqtime = explode("___", $w3all_nonce_reqtime_rand);
      // extract nonce and reqtime: check when (the time) the request has been done, so to avoid if too old
      $nonce_reqtime = explode("___", $w3all_nonce_reqtime_rand_meta['w3all_nonce_reqtime_rand']);
      $metareqtime = $nonce_reqtime[0];
      $metanonce = $nonce_reqtime[1];
      // the cURL request can be valid for 10 seconds
      if( time() > $metareqtime+10 ) { return; }

      $emails = '';
      foreach ( $del_ary as $e ){
       $emails .= "'".strtolower($e)."'".',';
      }

      $emails = substr($emails, 0, -1);
      $elist = 'LOWER(user_email) IN('.$emails.')';

      if (!function_exists('user_delete'))
      {
        include($this->phpbb_root_path . 'includes/functions_user.php');
        /**
         * Delete user(s) and their related data
         * @param string  $mode       Mode of posts deletion (retain|remove)
         * @param mixed   $user_ids     Either an array of integers or an integer
         * @param bool    $retain_username  True if username should be retained, false otherwise
         * @return bool
         //function user_delete($mode, $user_ids, $retain_username = true)
        */
       }

     $sql = "SELECT U.user_id
      FROM " . USERS_TABLE . "
      as U WHERE $elist";
      $res = $this->db->sql_query($sql);
       while ($row = $this->db->sql_fetchrow($res))
        {
         user_delete($delmode, $row['user_id'], $retain_username = true); // delete user
        }
       $this->db->sql_freeresult($res);
    }
  }  // END if( $phpBB_function_endpoint == 'user_delete' )


 }

  // ENDPOINTS from WordPress END
  //////////////////////////////////////

}
